@@ -1,6 +1,6 @@
 /* Uniform Interface to GMP.
 
-Copyright 2004-2017 Free Software Foundation, Inc.
+Copyright 2004-2019 Free Software Foundation, Inc.
 Contributed by the AriC and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
@@ -17,7 +17,7 @@ License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
-http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
+https://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
 
 #ifndef __GMPFR_GMP_H__
@@ -80,10 +80,7 @@ extern "C" {
 # error "Can't compute log2(GMP_NUMB_BITS)"
 #endif
 
-/* Remap names of internal mpn functions (for longlong.h).
-   Note: this should be made both with and without gmp build. */
-#undef  __clz_tab
-#define __clz_tab               mpfr_clz_tab
+
 
 /******************************************************
  ************* Define GMP Internal Interface  *********
@@ -234,6 +231,10 @@ typedef mp_limb_t UWtype;
 typedef unsigned int UHWtype;
 #define W_TYPE_SIZE GMP_NUMB_BITS
 
+/* Remap names of internal mpn functions (for longlong.h).  */
+#undef  __clz_tab
+#define __clz_tab               mpfr_clz_tab
+
 /* Use (4.0 * ...) instead of (2.0 * ...) to work around buggy compilers
    that don't convert ulong->double correctly (eg. SunOS 4 native cc).  */
 #undef MP_BASE_AS_DOUBLE
@@ -309,7 +310,7 @@ __MPFR_DECLSPEC void *mpfr_tmp_allocate (struct tmp_marker **,
                                          size_t);
 __MPFR_DECLSPEC void mpfr_tmp_free (struct tmp_marker *);
 
-/* Can be overriden at configure time. Useful for checking buffer overflow. */
+/* Can be overridden at configure time. Useful for checking buffer overflow. */
 #ifndef MPFR_ALLOCA_MAX
 # define MPFR_ALLOCA_MAX 16384
 #endif
