@@ -1,7 +1,7 @@
 /* mpfr_mul_d -- multiply a multiple precision floating-point number
                  by a machine double precision float
 
-Copyright 2007-2017 Free Software Foundation, Inc.
+Copyright 2007-2019 Free Software Foundation, Inc.
 Contributed by the AriC and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
@@ -18,7 +18,7 @@ License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
-http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
+https://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
 
 #include "mpfr-impl.h"
@@ -28,6 +28,7 @@ mpfr_mul_d (mpfr_ptr a, mpfr_srcptr b, double c, mpfr_rnd_t rnd_mode)
 {
   int inexact;
   mpfr_t d;
+  mp_limb_t tmp_man[MPFR_LIMBS_PER_DOUBLE];
   MPFR_SAVE_EXPO_DECL (expo);
 
   MPFR_LOG_FUNC
@@ -38,15 +39,14 @@ mpfr_mul_d (mpfr_ptr a, mpfr_srcptr b, double c, mpfr_rnd_t rnd_mode)
 
   MPFR_SAVE_EXPO_MARK (expo);
 
-  mpfr_init2 (d, IEEE_DBL_MANT_DIG);
+  MPFR_TMP_INIT1(tmp_man, d, IEEE_DBL_MANT_DIG);
   inexact = mpfr_set_d (d, c, rnd_mode);
-  MPFR_ASSERTN (inexact == 0);
+  MPFR_ASSERTD (inexact == 0);
 
-  mpfr_clear_flags ();
+  MPFR_CLEAR_FLAGS ();
   inexact = mpfr_mul (a, b, d, rnd_mode);
   MPFR_SAVE_EXPO_UPDATE_FLAGS (expo, __gmpfr_flags);
 
-  mpfr_clear(d);
   MPFR_SAVE_EXPO_FREE (expo);
   return mpfr_check_range (a, inexact, rnd_mode);
 }
