@@ -1,8 +1,8 @@
 /* mpfr_random2 -- Generate a positive random mpfr_t of specified size, with
    long runs of consecutive ones and zeros in the binary representation.
 
-Copyright 1999, 2001, 2002, 2003, 2004, 2006, 2007, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
-Contributed by the Arenaire and Caramel projects, INRIA.
+Copyright 1999, 2001-2004, 2006-2019 Free Software Foundation, Inc.
+Contributed by the AriC and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
 
@@ -18,7 +18,7 @@ License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
-http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
+https://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
 
 #include "mpfr-test.h"
@@ -30,6 +30,7 @@ http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
 #define BITS_PER_RANDCALL 32
 #endif
 
+/* exp is the maximum exponent in absolute value */
 void
 mpfr_random2 (mpfr_ptr x, mp_size_t size, mpfr_exp_t exp,
               gmp_randstate_t rstate)
@@ -97,7 +98,7 @@ mpfr_random2 (mpfr_ptr x, mp_size_t size, mpfr_exp_t exp,
               xp[ri--] = acc | (((mp_limb_t) 2 << bit_pos) - 1);
               bit_pos += GMP_NUMB_BITS;
               bit_pos -= nb;
-              acc = ((~(mp_limb_t) 1) << bit_pos) & GMP_NUMB_MASK;
+              acc = (~MPFR_LIMB_ONE) << bit_pos;
             }
           else
             {
@@ -137,8 +138,9 @@ mpfr_random2 (mpfr_ptr x, mp_size_t size, mpfr_exp_t exp,
 
   /* Generate random exponent.  */
   mpfr_rand_raw (&elimb, RANDS, GMP_NUMB_BITS);
-  exp = ABS (exp);
-  MPFR_SET_EXP (x, elimb % (2 * exp + 1) - exp);
+  MPFR_ASSERTN (exp >= 0 && exp <= MPFR_EMAX_MAX);
+  exp = (mpfr_exp_t) (elimb % (2 * exp + 1)) - exp;
+  MPFR_SET_EXP (x, exp);
 
   return ;
 }
