@@ -1,6 +1,6 @@
 /* Test file for mpfr_div_ui.
 
-Copyright 1999-2018 Free Software Foundation, Inc.
+Copyright 1999-2019 Free Software Foundation, Inc.
 Contributed by the AriC and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
@@ -17,7 +17,7 @@ License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
-http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
+https://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
 
 #include <float.h>
@@ -313,6 +313,7 @@ bug20180126 (void)
   /* This test expects that a limb fits in an unsigned long.
      One failing case from function bug20180126() in tdiv.c,
      for GMP_NUMB_BITS == 64. */
+#if ULONG_MAX > 4294967295
   if (GMP_NUMB_BITS == 64 && MPFR_LIMB_MAX <= ULONG_MAX)
     {
       mpfr_init2 (x, 133);
@@ -345,6 +346,7 @@ bug20180126 (void)
       mpfr_clear (y);
       mpfr_clear (z);
     }
+#endif
 }
 
 /* check corner cases where the round bit is located in the upper bit of r */
@@ -366,7 +368,9 @@ corner_cases (int n)
           mpfr_init2 (t, 2 * GMP_NUMB_BITS);
           for (i = 0; i < n; i++)
             {
-              u = randlimb ();
+              do
+                u = randlimb ();
+              while (u == 0);
               do
                 v = randlimb ();
               while (v <= MPFR_LIMB_HIGHBIT);
