@@ -1,7 +1,7 @@
-/* mpfr_tgamma -- test file for gamma function
+/* Test file for gamma function
 
-Copyright 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
-Contributed by the Arenaire and Caramel projects, INRIA.
+Copyright 2001-2019 Free Software Foundation, Inc.
+Contributed by the AriC and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
 
@@ -17,12 +17,9 @@ License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
-http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
+https://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
 
-#include <stdio.h>
-#include <stdlib.h>
-
 #include "mpfr-test.h"
 
 /* Note: there could be an incorrect test about suspicious overflow
@@ -111,9 +108,9 @@ special (void)
     {
       printf ("mpfr_lngamma("CHECK_X1") is wrong:\n"
               "expected ");
-      mpfr_print_binary (x); putchar ('\n');
+      mpfr_dump (x);
       printf ("got      ");
-      mpfr_print_binary (y); putchar ('\n');
+      mpfr_dump (y);
       exit (1);
     }
 
@@ -126,9 +123,9 @@ special (void)
     {
       printf ("mpfr_lngamma("CHECK_X2") is wrong:\n"
               "expected ");
-      mpfr_print_binary (x); putchar ('\n');
+      mpfr_dump (x);
       printf ("got      ");
-      mpfr_print_binary (y); putchar ('\n');
+      mpfr_dump (y);
       exit (1);
     }
 
@@ -153,8 +150,8 @@ special (void)
   if (mpfr_cmp (x, y))
     {
       printf ("Error in mpfr_gamma (120)\n");
-      printf ("Expected "); mpfr_print_binary (y); puts ("");
-      printf ("Got      "); mpfr_print_binary (x); puts ("");
+      printf ("Expected "); mpfr_dump (y);
+      printf ("Got      "); mpfr_dump (x);
       exit (1);
     }
 
@@ -192,6 +189,18 @@ special (void)
       exit (1);
     }
 
+  mpfr_set_prec (x, 2);
+  mpfr_set_prec (y, 2);
+  mpfr_set_ui (x, 2, MPFR_RNDN);
+  mpfr_clear_inexflag ();
+  mpfr_gamma (y, x, MPFR_RNDN);
+  if (mpfr_inexflag_p ())
+    {
+      printf ("Wrong inexact flag for gamma(2)\n");
+      printf ("expected 0, got 1\n");
+      exit (1);
+    }
+
   mpfr_clear (x);
   mpfr_clear (y);
 }
@@ -225,7 +234,7 @@ special_overflow (void)
   mpfr_set_prec (y, 29);
   mpfr_set_str (x, "-200000000.5", 10, MPFR_RNDN); /* exact */
   mpfr_gamma (y, x, MPFR_RNDN);
-  if (!(mpfr_zero_p (y) && MPFR_SIGN (y) < 0))
+  if (!(mpfr_zero_p (y) && MPFR_IS_NEG (y)))
     {
       printf ("Error for gamma(-200000000.5)\n");
       printf ("expected -0");
@@ -238,7 +247,7 @@ special_overflow (void)
   mpfr_set_prec (y, 53);
   mpfr_set_str (x, "-200000000.1", 10, MPFR_RNDN);
   mpfr_gamma (y, x, MPFR_RNDN);
-  if (!(mpfr_zero_p (y) && MPFR_SIGN (y) < 0))
+  if (!(mpfr_zero_p (y) && MPFR_IS_NEG (y)))
     {
       printf ("Error for gamma(-200000000.1), prec=53\n");
       printf ("expected -0");
@@ -352,7 +361,7 @@ special_overflow (void)
   mpfr_set_prec (y, 71);
   mpfr_set_str (x, "-200000000.1", 10, MPFR_RNDN);
   mpfr_gamma (y, x, MPFR_RNDN);
-  if (!(mpfr_zero_p (y) && MPFR_SIGN (y) < 0))
+  if (!(mpfr_zero_p (y) && MPFR_IS_NEG (y)))
     {
       printf ("Error for gamma (test 8)\n");
       printf ("expected "); mpfr_dump (x);
@@ -360,7 +369,7 @@ special_overflow (void)
       exit (1);
     }
 
-  set_emax (1073741823);
+  set_emax (1073741821);
   mpfr_set_prec (x, 29);
   mpfr_set_prec (y, 29);
   mpfr_set_str (x, "423786866", 10, MPFR_RNDN);
@@ -464,18 +473,87 @@ test20071231 (void)
   mpfr_clear (x);
 }
 
-/* bug found by Stathis, only occurs on 32-bit machines */
+/* bug found by Stathis in mpfr_gamma, only occurs on 32-bit machines;
+   the second test is for 64-bit machines. This bug reappeared due to
+   r8159. */
 static void
 test20100709 (void)
 {
-  mpfr_t x;
+  mpfr_t x, y, z;
+  int sign;
   int inex;
+  mpfr_exp_t emin;
 
   mpfr_init2 (x, 100);
+  mpfr_init2 (y, 32);
+  mpfr_init2 (z, 32);
   mpfr_set_str (x, "-4.6308260837372266e+07", 10, MPFR_RNDN);
+  mpfr_set_ui (y, 0, MPFR_RNDN);
+  mpfr_nextabove (y);
+  mpfr_log (y, y, MPFR_RNDD);
+  mpfr_const_log2 (z, MPFR_RNDU);
+  mpfr_sub (y, y, z, MPFR_RNDD); /* log(MIN/2) = log(MIN) - log(2) */
+  mpfr_lgamma (z, &sign, x, MPFR_RNDU);
+  MPFR_ASSERTN (sign == -1);
+  MPFR_ASSERTN (mpfr_less_p (z, y)); /* thus underflow */
   inex = mpfr_gamma (x, x, MPFR_RNDN);
-  MPFR_ASSERTN(MPFR_IS_ZERO(x) && MPFR_IS_NEG(x) && inex > 0);
+  MPFR_ASSERTN (MPFR_IS_ZERO(x) && MPFR_IS_NEG(x) && inex > 0);
   mpfr_clear (x);
+  mpfr_clear (y);
+  mpfr_clear (z);
+
+  /* Similar test for 64-bit machines (also valid with a 32-bit exponent,
+     but will not trigger the bug). */
+  emin = mpfr_get_emin ();
+  mpfr_set_emin (MPFR_EMIN_MIN);
+  mpfr_init2 (x, 100);
+  mpfr_init2 (y, 32);
+  mpfr_init2 (z, 32);
+  mpfr_set_str (x, "-90.6308260837372266e+15", 10, MPFR_RNDN);
+  mpfr_set_ui (y, 0, MPFR_RNDN);
+  mpfr_nextabove (y);
+  mpfr_log (y, y, MPFR_RNDD);
+  mpfr_const_log2 (z, MPFR_RNDU);
+  mpfr_sub (y, y, z, MPFR_RNDD); /* log(MIN/2) = log(MIN) - log(2) */
+  mpfr_lgamma (z, &sign, x, MPFR_RNDU);
+  MPFR_ASSERTN (sign == -1);
+  MPFR_ASSERTN (mpfr_less_p (z, y)); /* thus underflow */
+  inex = mpfr_gamma (x, x, MPFR_RNDN);
+  MPFR_ASSERTN (MPFR_IS_ZERO(x) && MPFR_IS_NEG(x) && inex > 0);
+  mpfr_clear (x);
+  mpfr_clear (y);
+  mpfr_clear (z);
+  mpfr_set_emin (emin);
+}
+
+/* bug found by Giridhar Tammana */
+static void
+test20120426 (void)
+{
+  mpfr_t xa, xb;
+  int i;
+  mpfr_exp_t emin;
+
+  mpfr_init2 (xa, 53);
+  mpfr_init2 (xb, 53);
+  mpfr_set_si_2exp (xb, -337, -1, MPFR_RNDN);
+  emin = mpfr_get_emin ();
+  mpfr_set_emin (-1073);
+  i = mpfr_gamma (xa, xb, MPFR_RNDN);
+  i = mpfr_subnormalize (xa, i, MPFR_RNDN); /* new ternary value */
+  mpfr_set_str (xb, "-9.5737343987585366746184749943e-304", 10, MPFR_RNDN);
+  if (!((i > 0) && (mpfr_cmp (xa, xb) == 0)))
+    {
+      printf ("Error in test20120426, i=%d\n", i);
+      printf ("expected ");
+      mpfr_dump (xb);
+      printf ("got      ");
+      mpfr_dump (xa);
+      exit (1);
+    }
+  mpfr_set_emin (emin);
+  mpfr_clear (xa);
+  mpfr_clear (xb);
 }
 
 static void
@@ -508,9 +586,9 @@ exprange (void)
       printf ("Error in exprange (test1)\n");
       printf ("x = ");
       mpfr_dump (x);
-      printf ("Expected inex1 = %d, flags1 = %u, ", SIGN (inex1), flags1);
+      printf ("Expected inex1 = %d, flags1 = %u, ", VSIGN (inex1), flags1);
       mpfr_dump (y);
-      printf ("Got      inex2 = %d, flags2 = %u, ", SIGN (inex2), flags2);
+      printf ("Got      inex2 = %d, flags2 = %u, ", VSIGN (inex2), flags2);
       mpfr_dump (z);
       exit (1);
     }
@@ -531,9 +609,9 @@ exprange (void)
       printf ("Error in exprange (test2)\n");
       printf ("x = ");
       mpfr_dump (x);
-      printf ("Expected inex1 = %d, flags1 = %u, ", SIGN (inex1), flags1);
+      printf ("Expected inex1 = %d, flags1 = %u, ", VSIGN (inex1), flags1);
       mpfr_dump (y);
-      printf ("Got      inex2 = %d, flags2 = %u, ", SIGN (inex2), flags2);
+      printf ("Got      inex2 = %d, flags2 = %u, ", VSIGN (inex2), flags2);
       mpfr_dump (z);
       exit (1);
     }
@@ -553,9 +631,9 @@ exprange (void)
       printf ("Error in exprange (test3)\n");
       printf ("x = ");
       mpfr_dump (x);
-      printf ("Expected inex1 = %d, flags1 = %u, ", SIGN (inex1), flags1);
+      printf ("Expected inex1 = %d, flags1 = %u, ", VSIGN (inex1), flags1);
       mpfr_dump (y);
-      printf ("Got      inex2 = %d, flags2 = %u, ", SIGN (inex2), flags2);
+      printf ("Got      inex2 = %d, flags2 = %u, ", VSIGN (inex2), flags2);
       mpfr_dump (z);
       exit (1);
     }
@@ -576,9 +654,9 @@ exprange (void)
       printf ("Error in exprange (test4)\n");
       printf ("x = ");
       mpfr_dump (x);
-      printf ("Expected inex1 = %d, flags1 = %u, ", SIGN (inex1), flags1);
+      printf ("Expected inex1 = %d, flags1 = %u, ", VSIGN (inex1), flags1);
       mpfr_dump (y);
-      printf ("Got      inex2 = %d, flags2 = %u, ", SIGN (inex2), flags2);
+      printf ("Got      inex2 = %d, flags2 = %u, ", VSIGN (inex2), flags2);
       mpfr_dump (z);
       exit (1);
     }
@@ -601,9 +679,9 @@ exprange (void)
       printf ("Error in exprange (test5)\n");
       printf ("x = ");
       mpfr_dump (x);
-      printf ("Expected inex1 = %d, flags1 = %u, ", SIGN (inex1), flags1);
+      printf ("Expected inex1 = %d, flags1 = %u, ", VSIGN (inex1), flags1);
       mpfr_dump (y);
-      printf ("Got      inex2 = %d, flags2 = %u, ", SIGN (inex2), flags2);
+      printf ("Got      inex2 = %d, flags2 = %u, ", VSIGN (inex2), flags2);
       mpfr_dump (z);
       exit (1);
     }
@@ -617,9 +695,9 @@ exprange (void)
       printf ("Error in exprange (test6)\n");
       printf ("x = ");
       mpfr_dump (x);
-      printf ("Expected inex1 = %d, flags1 = %u, ", SIGN (inex1), flags1);
+      printf ("Expected inex1 = %d, flags1 = %u, ", VSIGN (inex1), flags1);
       mpfr_dump (y);
-      printf ("Got      inex2 = %d, flags2 = %u, ", SIGN (inex2), flags2);
+      printf ("Got      inex2 = %d, flags2 = %u, ", VSIGN (inex2), flags2);
       mpfr_dump (z);
       exit (1);
     }
@@ -635,9 +713,9 @@ exprange (void)
       printf ("Error in exprange (test7)\n");
       printf ("x = ");
       mpfr_dump (x);
-      printf ("Expected inex1 = %d, flags1 = %u, ", SIGN (inex1), flags1);
+      printf ("Expected inex1 = %d, flags1 = %u, ", VSIGN (inex1), flags1);
       mpfr_dump (y);
-      printf ("Got      inex2 = %d, flags2 = %u, ", SIGN (inex2), flags2);
+      printf ("Got      inex2 = %d, flags2 = %u, ", VSIGN (inex2), flags2);
       mpfr_dump (z);
       exit (1);
     }
@@ -655,9 +733,9 @@ exprange (void)
       printf ("Error in exprange (test8)\n");
       printf ("x = ");
       mpfr_dump (x);
-      printf ("Expected inex1 = %d, flags1 = %u, ", SIGN (inex1), flags1);
+      printf ("Expected inex1 = %d, flags1 = %u, ", VSIGN (inex1), flags1);
       mpfr_dump (y);
-      printf ("Got      inex2 = %d, flags2 = %u, ", SIGN (inex2), flags2);
+      printf ("Got      inex2 = %d, flags2 = %u, ", VSIGN (inex2), flags2);
       mpfr_dump (z);
       exit (1);
     }
@@ -671,9 +749,9 @@ exprange (void)
       printf ("Error in exprange (test9)\n");
       printf ("x = ");
       mpfr_dump (x);
-      printf ("Expected inex1 = %d, flags1 = %u, ", SIGN (inex1), flags1);
+      printf ("Expected inex1 = %d, flags1 = %u, ", VSIGN (inex1), flags1);
       mpfr_dump (y);
-      printf ("Got      inex2 = %d, flags2 = %u, ", SIGN (inex2), flags2);
+      printf ("Got      inex2 = %d, flags2 = %u, ", VSIGN (inex2), flags2);
       mpfr_dump (z);
       exit (1);
     }
@@ -690,9 +768,9 @@ exprange (void)
       printf ("Error in exprange (test10)\n");
       printf ("x = ");
       mpfr_dump (x);
-      printf ("Expected inex1 = %d, flags1 = %u, ", SIGN (inex1), flags1);
+      printf ("Expected inex1 = %d, flags1 = %u, ", VSIGN (inex1), flags1);
       mpfr_dump (y);
-      printf ("Got      inex2 = %d, flags2 = %u, ", SIGN (inex2), flags2);
+      printf ("Got      inex2 = %d, flags2 = %u, ", VSIGN (inex2), flags2);
       mpfr_dump (z);
       exit (1);
     }
@@ -719,7 +797,7 @@ tiny_aux (int stop, mpfr_exp_t e)
   spm = 1;
   for (s = 0; s < 2; s++)
     {
-      RND_LOOP(r)
+      RND_LOOP_NO_RNDF (r)
         {
           mpfr_rnd_t rr = (mpfr_rnd_t) r;
           mpfr_exp_t exponent, emax;
@@ -758,12 +836,12 @@ tiny_aux (int stop, mpfr_exp_t e)
                 {
                   printf ("Error in tiny for s = %d, r = %s, emax = %"
                           MPFR_EXP_FSPEC "d%s\n  on ",
-                          s, mpfr_print_rnd_mode (rr), emax,
+                          s, mpfr_print_rnd_mode (rr), (mpfr_eexp_t) emax,
                           exponent > emax ? " (overflow)" : "");
                   mpfr_dump (x);
                   printf ("  expected inex = %2d, ", expected_inex);
                   mpfr_dump (z);
-                  printf ("  got      inex = %2d, ", SIGN (inex));
+                  printf ("  got      inex = %2d, ", VSIGN (inex));
                   mpfr_dump (y);
                   printf ("  expected flags = %u, got %u\n",
                           expected_flags, flags);
@@ -808,19 +886,203 @@ tiny (int stop)
     exit (1);
 }
 
+/* Test mpfr_gamma in precision p1 by comparing it with exp(lgamma(x))
+   computing with a working precision p2. Assume that x is not an
+   integer <= 2. */
+static void
+exp_lgamma (mpfr_t x, mpfr_prec_t p1, mpfr_prec_t p2)
+{
+  mpfr_t yd, yu, zd, zu;
+  int inexd, inexu, sign;
+  int underflow = -1, overflow = -1;  /* -1: we don't know */
+  int got_underflow, got_overflow;
+
+  if (mpfr_integer_p (x) && mpfr_cmp_si (x, 2) <= 0)
+    {
+      printf ("Warning! x is an integer <= 2 in exp_lgamma: ");
+      mpfr_out_str (stdout, 10, 0, x, MPFR_RNDN); putchar ('\n');
+      return;
+    }
+  mpfr_inits2 (p2, yd, yu, (mpfr_ptr) 0);
+  inexd = mpfr_lgamma (yd, &sign, x, MPFR_RNDD);
+  mpfr_set (yu, yd, MPFR_RNDN);  /* exact */
+  if (inexd)
+    mpfr_nextabove (yu);
+  mpfr_clear_flags ();
+  mpfr_exp (yd, yd, MPFR_RNDD);
+  if (! mpfr_underflow_p ())
+    underflow = 0;
+  if (mpfr_overflow_p ())
+    overflow = 1;
+  mpfr_clear_flags ();
+  mpfr_exp (yu, yu, MPFR_RNDU);
+  if (mpfr_underflow_p ())
+    underflow = 1;
+  if (! mpfr_overflow_p ())
+    overflow = 0;
+  if (sign < 0)
+    {
+      mpfr_neg (yd, yd, MPFR_RNDN);  /* exact */
+      mpfr_neg (yu, yu, MPFR_RNDN);  /* exact */
+      mpfr_swap (yd, yu);
+    }
+  /* yd < Gamma(x) < yu (strict inequalities since x != 1 and x != 2) */
+  mpfr_inits2 (p1, zd, zu, (mpfr_ptr) 0);
+  mpfr_clear_flags ();
+  inexd = mpfr_gamma (zd, x, MPFR_RNDD);  /* zd <= Gamma(x) < yu */
+  got_underflow = underflow == -1 ? -1 : !! mpfr_underflow_p ();
+  got_overflow = overflow == -1 ? -1 : !! mpfr_overflow_p ();
+  if (! mpfr_less_p (zd, yu) || inexd > 0 ||
+      got_underflow != underflow ||
+      got_overflow != overflow)
+    {
+      printf ("Error in exp_lgamma on x = ");
+      mpfr_out_str (stdout, 16, 0, x, MPFR_RNDN); putchar ('\n');
+      printf ("yu = ");
+      mpfr_dump (yu);
+      printf ("zd = ");
+      mpfr_dump (zd);
+      printf ("got inexd = %d, expected <= 0\n", inexd);
+      printf ("got underflow = %d, expected %d\n", got_underflow, underflow);
+      printf ("got overflow = %d, expected %d\n", got_overflow, overflow);
+      exit (1);
+    }
+  mpfr_clear_flags ();
+  inexu = mpfr_gamma (zu, x, MPFR_RNDU);  /* zu >= Gamma(x) > yd */
+  got_underflow = underflow == -1 ? -1 : !! mpfr_underflow_p ();
+  got_overflow = overflow == -1 ? -1 : !! mpfr_overflow_p ();
+  if (! mpfr_greater_p (zu, yd) || inexu < 0 ||
+      got_underflow != underflow ||
+      got_overflow != overflow)
+    {
+      printf ("Error in exp_lgamma on x = ");
+      mpfr_out_str (stdout, 16, 0, x, MPFR_RNDN); putchar ('\n');
+      printf ("yd = ");
+      mpfr_dump (yd);
+      printf ("zu = ");
+      mpfr_dump (zu);
+      printf ("got inexu = %d, expected >= 0\n", inexu);
+      printf ("got underflow = %d, expected %d\n", got_underflow, underflow);
+      printf ("got overflow = %d, expected %d\n", got_overflow, overflow);
+      exit (1);
+    }
+  if (mpfr_equal_p (zd, zu))
+    {
+      if (inexd != 0 || inexu != 0)
+        {
+          printf ("Error in exp_lgamma on x = ");
+          mpfr_out_str (stdout, 16, 0, x, MPFR_RNDN); putchar ('\n');
+          printf ("zd = zu, thus exact, but inexd = %d and inexu = %d\n",
+                  inexd, inexu);
+          exit (1);
+        }
+      MPFR_ASSERTN (got_underflow == 0);
+      MPFR_ASSERTN (got_overflow == 0);
+    }
+  else if (inexd == 0 || inexu == 0)
+    {
+      printf ("Error in exp_lgamma on x = ");
+          mpfr_out_str (stdout, 16, 0, x, MPFR_RNDN); putchar ('\n');
+          printf ("zd != zu, thus inexact, but inexd = %d and inexu = %d\n",
+                  inexd, inexu);
+          exit (1);
+    }
+  mpfr_clears (yd, yu, zd, zu, (mpfr_ptr) 0);
+}
+
+static void
+exp_lgamma_tests (void)
+{
+  mpfr_t x;
+  mpfr_exp_t emin, emax;
+  int i;
+
+  emin = mpfr_get_emin ();
+  emax = mpfr_get_emax ();
+  set_emin (MPFR_EMIN_MIN);
+  set_emax (MPFR_EMAX_MAX);
+
+  mpfr_init2 (x, 96);
+  for (i = 3; i <= 8; i++)
+    {
+      mpfr_set_ui (x, i, MPFR_RNDN);
+      exp_lgamma (x, 53, 64);
+      mpfr_nextbelow (x);
+      exp_lgamma (x, 53, 64);
+      mpfr_nextabove (x);
+      mpfr_nextabove (x);
+      exp_lgamma (x, 53, 64);
+    }
+  mpfr_set_str (x, "1.7", 10, MPFR_RNDN);
+  exp_lgamma (x, 53, 64);
+  mpfr_set_str (x, "-4.6308260837372266e+07", 10, MPFR_RNDN);
+  exp_lgamma (x, 53, 64);
+  mpfr_set_str (x, "-90.6308260837372266e+15", 10, MPFR_RNDN);
+  exp_lgamma (x, 53, 64);
+  /* The following test gives a large positive result < +Inf */
+  mpfr_set_str (x, "1.2b13fc45a92dea1@14", 16, MPFR_RNDN);
+  exp_lgamma (x, 53, 64);
+  /* Idem for a large negative result > -Inf */
+  mpfr_set_str (x, "-1.2b13fc45a92de81@14", 16, MPFR_RNDN);
+  exp_lgamma (x, 53, 64);
+  /* The following two tests trigger an endless loop in r8186
+     on 64-bit machines (64-bit exponent). The second one (due
+     to undetected overflow) is a direct consequence of the
+     first one, due to the call of Gamma(2-x) if x < 1. */
+  mpfr_set_str (x, "1.2b13fc45a92dec8@14", 16, MPFR_RNDN);
+  exp_lgamma (x, 53, 64);
+  mpfr_set_str (x, "-1.2b13fc45a92dea8@14", 16, MPFR_RNDN);
+  exp_lgamma (x, 53, 64);
+  /* Similar tests (overflow threshold) for 32-bit machines. */
+  mpfr_set_str (x, "2ab68d8.657542f855111c61", 16, MPFR_RNDN);
+  exp_lgamma (x, 12, 64);
+  mpfr_set_str (x, "-2ab68d6.657542f855111c61", 16, MPFR_RNDN);
+  exp_lgamma (x, 12, 64);
+  /* The following test is an overflow on 32-bit and 64-bit machines.
+     Revision r8189 fails on 64-bit machines as the flag is unset. */
+  mpfr_set_str (x, "1.2b13fc45a92ded8@14", 16, MPFR_RNDN);
+  exp_lgamma (x, 53, 64);
+  /* On the following tests, with r8196, one gets an underflow on
+     32-bit machines, while a normal result is expected (see FIXME
+     in gamma.c:382). */
+  mpfr_set_str (x, "-2ab68d6.657542f855111c6104", 16, MPFR_RNDN);
+  exp_lgamma (x, 12, 64);  /* failure on 32-bit machines */
+  mpfr_set_str (x, "-12b13fc45a92deb.1c6c5bc964", 16, MPFR_RNDN);
+  exp_lgamma (x, 12, 64);  /* failure on 64-bit machines */
+  mpfr_clear (x);
+
+  set_emin (emin);
+  set_emax (emax);
+}
+
 int
 main (int argc, char *argv[])
 {
   tests_start_mpfr ();
 
+  if (argc == 3) /* tgamma x prec: print gamma(x) to prec bits */
+    {
+      mpfr_prec_t p = atoi (argv[2]);
+      mpfr_t x;
+      mpfr_init2 (x, p);
+      mpfr_set_str (x, argv[1], 10, MPFR_RNDN);
+      mpfr_gamma (x, x, MPFR_RNDN);
+      mpfr_out_str (stdout, 10, 0, x, MPFR_RNDN);
+      printf ("\n");
+      mpfr_clear (x);
+      return 0;
+    }
+
   special ();
   special_overflow ();
   exprange ();
   tiny (argc == 1);
-  test_generic (2, 100, 2);
+  test_generic (MPFR_PREC_MIN, 100, 2);
   gamma_integer ();
   test20071231 ();
   test20100709 ();
+  test20120426 ();
+  exp_lgamma_tests ();
 
   data_check ("data/gamma", mpfr_gamma, "mpfr_gamma");
 
