@@ -1,6 +1,6 @@
 /* Test file for mpfr_set_z_2exp.
 
-Copyright 1999, 2001-2017 Free Software Foundation, Inc.
+Copyright 1999, 2001-2019 Free Software Foundation, Inc.
 Contributed by the AriC and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
@@ -17,13 +17,9 @@ License for more details.
 
 You should have received a copy of the GNU Lesser General Public License
 along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
-http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
+https://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <limits.h>
-
 #include "mpfr-test.h"
 
 static mpfr_exp_t
@@ -62,6 +58,14 @@ check0 (void)
           exit (1);
         }
     }
+
+  /* coverage test for huge exponent */
+  mpz_setbit (y, GMP_NUMB_BITS);
+  mpfr_clear_flags ();
+  inexact = mpfr_set_z_2exp (x, y, mpfr_get_emax_max(), MPFR_RNDN);
+  MPFR_ASSERTN(inexact > 0);
+  MPFR_ASSERTN(mpfr_inf_p (x) && mpfr_sgn (x) > 0);
+  MPFR_ASSERTN(mpfr_overflow_p ());
   mpfr_clear(x);
   mpz_clear(y);
 }
